package ca.recrown.islandsurvivalcraft.interaction.commands.runnables;

import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import org.bukkit.Bukkit;
import org.bukkit.Color;
import org.bukkit.Location;
import org.bukkit.Particle;
import org.bukkit.World;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import ca.recrown.islandsurvivalcraft.IslandSurvivalCraft;
import ca.recrown.islandsurvivalcraft.utilities.datatypes.Point2;
import ca.recrown.islandsurvivalcraft.world.WorldInfo;
import ca.recrown.islandsurvivalcraft.world.Information.IslandInformation;

public class HighlightIslandCommand implements CommandRunnable {
    private boolean requiresInit = true;
    private final Particle.DustOptions EDGE_DUST_OPTIONS = new Particle.DustOptions(Color.RED, 1f);
    private final Particle.DustOptions ORIGIN_DUST_OPTIONS = new Particle.DustOptions(Color.GREEN, 1f);
    private final int PARTICLE_AMOUNT = 1;
    private final double OFFSET_X = 0.0d, OFFSET_Y = 6.0d, OFFSET_Z = 0.0d;
    private final double EXTRA = 0d;
    HashSet<Player> playersHighlighting = new HashSet<>();
    LinkedList<Player> waitingList = new LinkedList<>();

    @Override
    public String getDescription() {
        return "Displays particle effects at edge of islands.";
    }

    @Override
    public boolean invoke(CommandSender sender, String[] args) {
        if (!(sender instanceof Player)) {
            sender.sendMessage("This command can only be run as a player!");
            return false;
        }
        Player player = (Player) sender;
        if (args[0].isEmpty()) {
            return false;
        }
        if (args[0].toLowerCase().equals("start")) {
            if (playersHighlighting.add(player)) {
                waitingList.add(player);
                sender.sendMessage("You are now highlighting islands.");
            } else {
                sender.sendMessage("You are already highlighting islands.");
            }
        } else if (args[0].toLowerCase().equals("stop")) {
            if (playersHighlighting.remove(player)) {
                sender.sendMessage("You are no longer highlighting islands.");
            } else {
                sender.sendMessage("You weren't highlighting islands.");
            }
        } else {
            return false;
        }
        return true;
    }

    @Override
    public void initialize(IslandSurvivalCraft islandsurvivalcraft) {
        requiresInit = false;
        Bukkit.getScheduler().scheduleSyncRepeatingTask(islandsurvivalcraft, new Runnable() {
            @Override
            public void run() {
                Player player = waitingList.poll();
                if (player == null || !player.isOnline() || !playersHighlighting.contains(player)) {
                    playersHighlighting.remove(player);
                    return;
                }
                World world = player.getWorld();
                WorldInfo worldInfo = islandsurvivalcraft.getWorldInfoManager().retrieve(world.getName());
                Location playerLocation = player.getLocation();
                int playerX = playerLocation.getBlockX(), playerY = playerLocation.getBlockY(), playerZ = playerLocation.getBlockZ();
                Point2 playerCoords = new Point2(playerX, playerZ);
                IslandInformation islandInfo = worldInfo.getIslandInfoManager().getIslandInformation(playerCoords);
                if (islandInfo != null) {
                    Set<Point2> islandBorder = islandInfo.getEdgeCoordinates();
                    Iterator<Point2> islandborderIterable = islandBorder.iterator();
                    for (int i = 0; i < islandBorder.size(); i+= 2) {
                        Point2 current = islandborderIterable.next();
                        spawnParticle(world, current.x, playerY, current.y, EDGE_DUST_OPTIONS);
                        islandborderIterable.next();
                    }
                    spawnParticle(world, islandInfo.getIslandOrigin().x, playerY, islandInfo.getIslandOrigin().y, ORIGIN_DUST_OPTIONS);
                }
                waitingList.add(player);
            }
        }, 0, 10);

    }

    private void spawnParticle(World world, double x, double y, double z, Particle.DustOptions dustOptions) {
        world.spawnParticle(Particle.REDSTONE, x, y, z, PARTICLE_AMOUNT, OFFSET_X, OFFSET_Y, OFFSET_Z, EXTRA, dustOptions, false);
    }

    @Override
    public String getDetailedDescription() {
        return "A helpful debugging command to analyze what the plugin considers an island by displaying particle effects around the player that is a part of the island. The <start | stop> argument is required to dicate whether or not to start the highlighting or stop it.";
    }

    @Override
    public boolean requiresInitialization() {
        return requiresInit;
    }
}