package ca.recrown.islandsurvivalcraft.interaction.commands.runnables;

import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import ca.recrown.islandsurvivalcraft.IslandSurvivalCraft;
import ca.recrown.islandsurvivalcraft.world.IslandWorldMap;

public class ValueRunnable implements CommandRunnable {
    private IslandSurvivalCraft isc;

    @Override
    public String getDescription() {
        return "Gives the world value this plugin is using.";
    }

    @Override
    public String getDetailedDescription() {
        return "Player only command. Usage: \"IslandSurvivalCraft value [x] [z]\". Gives the current world value at the given x and z coordinates. If the two coordinates are not provided, player location is used.";
    }

    @Override
    public boolean invoke(CommandSender sender, String[] args) {
        if (!(sender instanceof Player)) return false;
        Player p = (Player) sender;
        int worldX = 0, worldZ = 0;
        if (args.length > 0) {
            if (args.length != 2) return false;
            try {
                worldX = Integer.valueOf(args[0]);
                worldZ = Integer.valueOf(args[1]);
            } catch (NumberFormatException e) {
                return false;
            }
        } else {
            worldX = p.getLocation().getBlockX();
            worldZ = p.getLocation().getBlockZ();

        }
        
        IslandWorldMap map = isc.getWorldInfoManager().retrieve(p.getWorld().getName()).getIslandMap();
        p.sendMessage(String.format("world: %s, World value (%d, %d): %s", 
                                    p.getWorld().getName(), 
                                    worldX, worldZ, 
                                    map.getWorldValue(worldX, worldZ)));

        p.sendMessage(String.format("Terrain: \nIsland: %b, Shallow: %b, Hill: %b, Land: %b, Shore: %b, Edge: %b, Transitional: %b, Deep: %b", 
                                    map.isIsland(worldX, worldZ), 
                                    map.isShallowPortion(worldX, worldZ), 
                                    map.isHill(worldX, worldZ), 
                                    map.isLand(worldX, worldZ), 
                                    map.isShore(worldX, worldZ),
                                    map.isEdgeOfIsland(worldX, worldZ),
                                    map.isTransitional(worldX, worldZ),
                                    map.isDeep(worldX, worldZ)));
        return true;
    }

    @Override
    public void initialize(IslandSurvivalCraft islandSurvivalCraft) {
        isc = islandSurvivalCraft;
    }

    @Override
    public boolean requiresInitialization() {
        return isc == null;
    }
    
}