package ca.recrown.islandsurvivalcraft.interaction.items;

import org.bukkit.Bukkit;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.Event.Result;
import org.bukkit.event.block.Action;
import org.bukkit.event.inventory.CraftItemEvent;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import ca.recrown.islandsurvivalcraft.IslandSurvivalCraft;

public class VariedItemManager implements Listener {
    private final IslandSurvivalCraft islandSurvivalCraft;

    public VariedItemManager(IslandSurvivalCraft islandSurvivalCraft) {
        this.islandSurvivalCraft = islandSurvivalCraft;
    }

    public void loadAllItems() {
        RegisteredVariedItem[] items = RegisteredVariedItem.values();
        for (int i = 0; i < items.length; i++) {
            if (items[i].getVariedItem().needInitialization()) items[i].getVariedItem().initialize(islandSurvivalCraft);
            islandSurvivalCraft.getServer().addRecipe(items[i].getVariedItem().getRecipe());
        }
    }

    public void unloadAllItems() {
        RegisteredVariedItem[] items = RegisteredVariedItem.values();
        for (int i = 0; i < items.length; i++) {
            Bukkit.removeRecipe(items[i].getVariedItem().getNamespacedKey());
        }
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onItemCrafted(CraftItemEvent e) {
        ItemStack item = e.getCurrentItem();
        VariedItem variedItem = getVariedItemFromItemStack(item);
        if (variedItem != null) {
            variedItem.onItemCrafted(item, e.getWhoClicked());
        }

    }

    @EventHandler(priority = EventPriority.HIGHEST)
    public void onPlayerInteraction(PlayerInteractEvent e) {
        if (e.useItemInHand() == Result.DENY || (e.getAction() != Action.RIGHT_CLICK_AIR && e.getAction() != Action.RIGHT_CLICK_BLOCK)) return;
        ItemStack item = e.getItem();
        VariedItem variedItem = getVariedItemFromItemStack(item);
        if (variedItem != null) {
            if(variedItem.onItemRightClicked(item, e.getPlayer(), e.getHand(), e.getClickedBlock(), e.getBlockFace())) {
                e.setCancelled(true);
                e.setUseItemInHand(Result.DENY);
            }
        }
    }

    

    private VariedItem getVariedItemFromItemStack(ItemStack item) {
        if (item == null) return null;
        ItemMeta meta = item.getItemMeta();
        RegisteredVariedItem[] items = RegisteredVariedItem.values();
        for (int i = 0; i < items.length; i++) {
            VariedItem curr = items[i].getVariedItem();
            String identifier = meta.getPersistentDataContainer().get(curr.getNamespacedKey(), curr.getIdentifier());
            if (identifier != null) {
                if (curr.getIdentifier().getID().equals(identifier)) {
                    return curr;
                }
            }
        }

        return null;
    }
}