package ca.recrown.islandsurvivalcraft.world.Information;

import java.util.HashSet;
import java.util.concurrent.ConcurrentHashMap;

import ca.recrown.islandsurvivalcraft.utilities.GeneralUtilities;
import ca.recrown.islandsurvivalcraft.utilities.datatypes.Point2;
import ca.recrown.islandsurvivalcraft.world.IslandWorldMap;

public class IslandInformationManager {
    private final IslandWorldMap islandMap;
    private final ConcurrentHashMap<Point2, HashSet<IslandInformation>> islandSets = new ConcurrentHashMap<>();

    public IslandInformationManager(IslandWorldMap islandWorldMap) {
        this.islandMap = islandWorldMap;
    }

    public void loadChunkIslandInformation(Point2 chunkCoords) {
        Point2[] surrounding = new Point2[4];
        surrounding[0] = chunkCoords.shift(1, 0);
        surrounding[1] = chunkCoords.shift(-1, 0);
        surrounding[2] = chunkCoords.shift(0, -1);
        surrounding[3] = chunkCoords.shift(0, 1);
        HashSet<Point2> checkedCoordinates = new HashSet<>();
        for (Point2 chunk : surrounding) {
            HashSet<IslandInformation> chunkIslandSet = getChunkIslandInformationSet(chunk);
            for (IslandInformation islandInformation : chunkIslandSet) {
                if (islandInformation.isIslandInChunk(chunkCoords)) {
                    checkedCoordinates.addAll(islandInformation.getIslandCoordinates());
                }
            }
        }


        for (int x = 0; x < GeneralUtilities.CHUNK_SIZE; x++) {
            for (int z = 0; z < GeneralUtilities.CHUNK_SIZE; z++) {
                int worldX = chunkCoords.x * GeneralUtilities.CHUNK_SIZE + x;
                int worldZ = chunkCoords.y * GeneralUtilities.CHUNK_SIZE + z;
                if (islandMap.isIsland(worldX, worldZ)) {
                    IslandInformationBuilder infoBuilder = new IslandInformationBuilder();
                    Point2 origin = islandMap.findIslandOrigin(worldX, worldZ, (p) -> {
                        if (!checkedCoordinates.add(p)) return true;
                        infoBuilder.addCoordinate(p, islandMap.isEdgeOfIsland(p.x, p.y));
                        infoBuilder.addChunk(GeneralUtilities.worldToChunkCoordinates(p));
                        return false;
                    });
                    if (origin != null) {
                        IslandInformation islandInfo = infoBuilder.build(origin);
                        for (Point2 chunk : infoBuilder.getChunksUsed()) {
                            getChunkIslandInformationSet(chunk).add(islandInfo);
                        }
                    }
                }
            }
        }
    }
    
    public void unloadChunkIslandInformation(Point2 chunkCoords) {
        islandSets.remove(chunkCoords);
    }

    public HashSet<IslandInformation> getChunkIslandInformationSet(Point2 chunkCoords) {
        if (!islandSets.containsKey(chunkCoords)) islandSets.put(chunkCoords, new HashSet<>());
        return islandSets.get(chunkCoords);
    }

    public IslandInformation getIslandInformation(Point2 coords) {
        HashSet<IslandInformation> chunkIslandSet = getChunkIslandInformationSet(GeneralUtilities.worldToChunkCoordinates(coords));
        for (IslandInformation islandInformation : chunkIslandSet) {
            if (islandInformation.isWithinIsland(coords)) return islandInformation;
        }
        return null;
    }
}